@@ -3,12 +3,15 @@ from __future__ import annotations
 __all__ = (
     "NeptuneScaleError",
     "NeptuneScaleWarning",
-    "NeptuneOperationsQueueMaxSizeExceeded",
+    "NeptuneUnableToLogData",
     "NeptuneUnauthorizedError",
+    "NeptuneBadRequestError",
     "NeptuneInvalidCredentialsError",
     "NeptuneUnexpectedError",
     "NeptuneConnectionLostError",
     "NeptuneUnableToAuthenticateError",
+    "NeptuneFileUploadError",
+    "NeptuneFileUploadTemporaryError",
     "NeptuneRetryableError",
     "NeptuneUnexpectedResponseError",
     "NeptuneInternalServerError",
@@ -34,14 +37,23 @@ __all__ = (
     "NeptuneStringValueExceedsSizeLimit",
     "NeptuneStringSetExceedsSizeLimit",
     "NeptuneSynchronizationStopped",
-    "NeptuneAsyncLagThresholdExceeded",
+    "NeptuneFileMetadataExceedsSizeLimit",
     "NeptuneProjectNotProvided",
     "NeptuneApiTokenNotProvided",
     "NeptuneTooManyRequestsResponseError",
     "NeptunePreviewStepNotAfterLastCommittedStep",
+    "NeptuneDatabaseConflict",
+    "NeptuneLocalStorageInUnsupportedVersion",
+    "NeptuneHistogramBinEdgesContainNaN",
+    "NeptuneHistogramTooManyBins",
+    "NeptuneHistogramBinEdgesNotIncreasing",
+    "NeptuneHistogramValuesLengthMismatch",
 )
 
-from typing import Any
+from typing import (
+    Any,
+    Optional,
+)
 
 from neptune_scale.util.styles import (
     STYLES,
@@ -50,7 +62,7 @@ from neptune_scale.util.styles import (
 
 
 class NeptuneScaleError(Exception):
-    message = "An error occurred in the Neptune Scale client."
+    message = "An error occurred in the Neptune client."
 
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         ensure_style_detected()
@@ -59,7 +71,7 @@ class NeptuneScaleError(Exception):
 
 
 class NeptuneScaleWarning(Warning):
-    message = "A warning occurred in the Neptune Scale client."
+    message = "A warning occurred in the Neptune client."
 
     def __init__(self, *args: Any, **kwargs: Any) -> None:
         ensure_style_detected()
@@ -86,35 +98,76 @@ NeptuneBadRequestError({status_code}): {reason}
 
 
 class NeptuneSynchronizationStopped(NeptuneScaleError):
-    message = "Internal synchronization process was stopped."
-
-
-class NeptuneOperationsQueueMaxSizeExceeded(NeptuneScaleError):
     message = """
 {h1}
-NeptuneOperationsQueueMaxSizeExceeded: The amount of data being logged is higher than processing capacity.
+NeptuneSynchronizationStopped: The background synchronization process has stopped unexpectedly.
 {end}
 
-The synchronization is paused until the queue size drops below the maximum.
-
-To resolve this issue, consider the following:
-    - Throttle the rate of operations
-    - Cautiously increase the queue size through the `max_queue_size` argument.
-        Note: To ensure that memory usage remains within acceptable limits,
-        closely monitor your system's memory consumption.
+Your data is still being saved locally. You can manually synchronize it with the
+Neptune backend later by running `neptune sync`.
 """
 
 
+class NeptuneUnableToLogData(NeptuneScaleError):
+    message = """
+{h1}
+NeptuneUnableToLogData: An error occurred, preventing Neptune from logging your data.
+{end}
+{reason}
+"""
+
+    def __init__(self, reason: str = "", **kwargs: Any) -> None:
+        super().__init__(reason=reason, **kwargs)
+
+
+class NeptuneFloatValueNanInfUnsupported(NeptuneUnableToLogData):
+    message = """
+{h1}
+NeptuneFloatValueNanInfUnsupported: metric `{metric}` at step `{step}` has non-finite value of `{value}`.
+{end}
+Float series do not support logging NaN and Inf values. You can only log NaN and Inf as single config values.
+
+You can configure Neptune to skip non-finite metric values by setting the `NEPTUNE_SKIP_NON_FINITE_METRICS`
+environment variable to `True`.
+
+For details, see https://docs.neptune.ai/log_configs
+"""
+
+    def __init__(self, metric: str = "", step: Optional[float | int] = None, value: Any = None) -> None:
+        super().__init__(metric=metric, step=step, value=value)
+
+
+class GenericFloatValueNanInfUnsupported(NeptuneFloatValueNanInfUnsupported):
+    message = """
+{h1}
+NeptuneFloatValueNanInfUnsupported: Float series do not support logging NaN and Inf values. You can only log NaN and Inf as single config values.
+{end}
+
+You can configure Neptune to skip non-finite metric values by setting the `NEPTUNE_SKIP_NON_FINITE_METRICS`
+environment variable to `True`.
+
+For details, see https://docs.neptune.ai/log_configs
+"""
+
+    def __init__(self, *_args: Any) -> None:
+        # Because this class defines its own message, the argument values to super().__init__ don't matter here.
+        # However, we still need to accept *args, for unpickling to work properly. At the end of inheritance chain,
+        # NeptuneScaleError calls Exception.__init__() with the `message` argument. This gets saved by pickle
+        # and later used when unpickling the exception object, which would fail if __init__ didn't accept any
+        # positional args. Our message is fixed, so this works OK.
+        super().__init__(metric="", step=None, value=None)
+
+
 class NeptuneUnauthorizedError(NeptuneScaleError):
     message = """
 {h1}
 NeptuneUnauthorizedError: You don't have permission to access the given resource.
 {end}
     - Verify that your API token is correct. To find your API token:
-        - Log in to Neptune Scale and open the user menu.
+        - Log in to Neptune and open the user menu.
         - If your workspace uses service accounts, ask the project owner to provide the token.
 
-    - Verify that the provided project name is correct.
+    - Verify that the provided project name is correct and the project exists.
       The correct project name should look like this: {correct}WORKSPACE_NAME/PROJECT_NAME{end}
       It has two parts:
           - {correct}WORKSPACE_NAME{end}: can be your username or your organization name
@@ -130,7 +183,7 @@ class NeptuneInvalidCredentialsError(NeptuneScaleError):
 NeptuneInvalidCredentialsError: The provided API token is invalid.
 {end}
 
-Make sure you copied your API token while logged in to Neptune Scale.
+Make sure you copied your API token while logged in to Neptune.
 If your workspace uses service accounts, ask the project owner for the token.
 
 There are two options to provide the API token:
@@ -154,14 +207,14 @@ and omit the {bold}api_token{end} argument from the {bold}Run{end} constructor:
 {h2}Option 2: Run argument{end}
 Pass the token to the {bold}Run{end} constructor via the {bold}api_token{end} argument:
     {python}neptune_scale.Run(project="WORKSPACE_NAME/PROJECT_NAME", api_token="YOUR_API_TOKEN"){end}
-For help, see https://docs-beta.neptune.ai/setup
+For help, see https://docs.neptune.ai/setup
 """
 
 
 class NeptuneUnexpectedError(NeptuneScaleError):
     message = """
 {h1}
-NeptuneUnexpectedError: An unexpected error occurred in the Neptune Scale client.
+NeptuneUnexpectedError: An unexpected error occurred in the Neptune client.
 {end}
 Reason: `{reason}`
 
@@ -212,7 +265,7 @@ This is a temporary problem. If the problem persists, please contact us at suppo
 """
 
 
-class NeptuneUnableToAuthenticateError(NeptuneScaleError):
+class NeptuneUnableToAuthenticateError(NeptuneRetryableError):
     message = """
 {h1}
 ----NeptuneUnableToAuthenticateError: The client was unable to authenticate with the Neptune server.
@@ -221,16 +274,36 @@ Ensure that your API token is correct.
 """
 
 
-class NeptuneProjectNotFound(NeptuneScaleError):
+class NeptuneFileUploadError(NeptuneScaleError):
+    message = """
+{h1}
+----NeptuneFileUploadError: An unrecoverable error occurred during file upload
+{end}
+"""
+
+
+class NeptuneFileUploadTemporaryError(NeptuneRetryableError):
+    message = """
+{h1}
+----NeptuneFileUploadTemporaryError: A temporary error occurred during file upload
+{end}
+"""
+
+
+class NeptuneProjectError(NeptuneScaleError):
+    pass
+
+
+class NeptuneProjectNotFound(NeptuneProjectError):
     message = """
 {h1}
 NeptuneProjectNotFound: Either the project hasn't been created yet or the name is incorrect.
 {end}
-For help, see https://docs-beta.neptune.ai/setup
+For help, see https://docs.neptune.ai/setup
 """
 
 
-class NeptuneProjectInvalidName(NeptuneScaleError):
+class NeptuneProjectInvalidName(NeptuneProjectError):
     message = """
 {h1}
 NeptuneProjectInvalidName: Project name is either empty or too long.
@@ -238,7 +311,7 @@ NeptuneProjectInvalidName: Project name is either empty or too long.
 """
 
 
-class NeptuneProjectAlreadyExists(NeptuneScaleError):
+class NeptuneProjectAlreadyExists(NeptuneProjectError):
     message = """
 {h1}
 NeptuneProjectAlreadyExists: A project with the provided name or project key already exists.
@@ -246,7 +319,11 @@ NeptuneProjectAlreadyExists: A project with the provided name or project key alr
 """
 
 
-class NeptuneRunNotFound(NeptuneScaleError):
+class NeptuneRunError(NeptuneScaleError):
+    pass
+
+
+class NeptuneRunNotFound(NeptuneRunError):
     message = """
 {h1}
 ----NeptuneRunNotFound: Run not found.
@@ -264,12 +341,12 @@ If you wanted to resume an existing run, include the argument `resume=True`.
 """
 
 
-class NeptuneRunConflicting(NeptuneScaleError):
+class NeptuneRunConflicting(NeptuneRunError):
     message = """
 {h1}
 NeptuneRunConflicting: Run with specified `run_id` already exists, but has a different `fork_run_id` parameter.
 {end}
-For forking instructions, see https://docs-beta.neptune.ai/fork_experiment
+For forking instructions, see https://docs.neptune.ai/fork_experiment
 """
 
 
@@ -278,11 +355,11 @@ class NeptuneRunForkParentNotFound(NeptuneScaleWarning):
 {h1}
 ----NeptuneRunForkParentNotFound: The provided parent run does not exist.
 {end}
-For forking instructions, see https://docs-beta.neptune.ai/fork_experiment
+For forking instructions, see https://docs.neptune.ai/fork_experiment
 """
 
 
-class NeptuneRunInvalidCreationParameters(NeptuneScaleError):
+class NeptuneRunInvalidCreationParameters(NeptuneRunError):
     message = """
 {h1}
 NeptuneRunInvalidCreationParameters: Run creation parameters rejected by the server.
@@ -324,7 +401,7 @@ NeptuneAttributePathNonWritable: Attribute is not writable.
 {end}
 You could be trying to overwrite a read-only attribute. Note that most of the "sys/*" attributes are read-only.
 
-For details, see https://docs-beta.neptune.ai/sys
+For details, see https://docs.neptune.ai/sys
 """
 
 
@@ -333,7 +410,7 @@ class NeptuneAttributeTypeUnsupported(NeptuneScaleError):
 {h1}
 NeptuneAttributeTypeUnsupported: the provided attribute type is not supported by Neptune.
 {end}
-For supported types, see https://docs-beta.neptune.ai/attribute_types
+For supported types, see https://docs.neptune.ai/attribute_types
 """
 
 
@@ -351,7 +428,7 @@ class NeptuneSeriesPointDuplicate(NeptuneScaleWarning):
 {h1}
 NeptuneSeriesPointDuplicate: The exact same data point (value + step pair) was already logged for this series.
 {end}
-For help, see https://docs-beta.neptune.ai/log_metrics
+For help, see https://docs.neptune.ai/log_metrics
 """
 
 
@@ -364,7 +441,7 @@ This can be caused by either:
 - The step of a series value is smaller than the most recently logged step for this series
 - the step is exactly the same but the value is different
 
-For help, see https://docs-beta.neptune.ai/log_metrics
+For help, see https://docs.neptune.ai/log_metrics
 """
 
 
@@ -373,7 +450,7 @@ class NeptuneSeriesStepNotAfterForkPoint(NeptuneScaleError):
 {h1}
 NeptuneSeriesStepNotAfterForkPoint: The series value must be greater than the step specified by the `fork_step` argument.
 {end}
-For help, see https://docs-beta.neptune.ai/fork_experiment
+For help, see https://docs.neptune.ai/fork_experiment
 """
 
 
@@ -382,34 +459,7 @@ class NeptuneSeriesTimestampDecreasing(NeptuneScaleError):
 {h1}
 NeptuneSeriesTimestampDecreasing: The timestamp of a series value is less than the most recently logged value.
 {end}
-Existing timestamps are allowed. For help, see https://docs-beta.neptune.ai/log_metrics
-"""
-
-
-class NeptuneFloatValueNanInfUnsupported(NeptuneScaleError):
-    message = """
-{h1}
-NeptuneFloatValueNanInfUnsupported: metric `{metric}` at step `{step}` has non-finite value of `{value}`.
-{end}
-Float series do not support logging NaN and Inf values. You can only log NaN and Inf as single config values.
-
-You can configure Neptune to skip non-finite metric values by setting the `NEPTUNE_SKIP_NON_FINITE_METRICS`
-environment variable to `True`.
-
-For details, see https://docs-beta.neptune.ai/log_configs
-"""
-
-
-class GenericFloatValueNanInfUnsupported(NeptuneFloatValueNanInfUnsupported):
-    message = """
-{h1}
-NeptuneFloatValueNanInfUnsupported: Float series do not support logging NaN and Inf values. You can only log NaN and Inf as single config values.
-{end}
-
-You can configure Neptune to skip non-finite metric values by setting the `NEPTUNE_SKIP_NON_FINITE_METRICS`
-environment variable to `True`.
-
-For details, see https://docs-beta.neptune.ai/log_configs
+Existing timestamps are allowed. For help, see https://docs.neptune.ai/log_metrics
 """
 
 
@@ -429,12 +479,11 @@ NeptuneStringSetExceedsSizeLimit: String Set value is too long. Maximum length i
 """
 
 
-class NeptuneAsyncLagThresholdExceeded(NeptuneScaleError):
+class NeptuneFileMetadataExceedsSizeLimit(NeptuneScaleError):
     message = """
 {h1}
-NeptuneAsyncLagThresholdExceeded: Neptune is experiencing a high delay in synchronizing data.
+NeptuneFileMetadataExceedsSizeLimit: File metadata is too long. Maximum length is 4KB.
 {end}
-This is a temporary problem. If the problem persists, please contact us at support@neptune.ai
 """
 
 
@@ -446,7 +495,7 @@ NeptuneProjectNotProvided: The project name was not provided.
 Make sure to specify the project name in the `project` parameter of the `Run`
 constructor or with the `NEPTUNE_PROJECT` environment variable.
 
-For instructions, see https://docs-beta.neptune.ai/setup
+For instructions, see https://docs.neptune.ai/setup
 
 """
 
@@ -459,7 +508,7 @@ NeptuneApiTokenNotProvided: The Neptune API token was not provided.
 Make sure to specify the API token in the `api_token` parameter of the `Run`
 constructor or with the `NEPTUNE_API_TOKEN` environment variable.
 
-For instructions, see https://docs-beta.neptune.ai/api_token
+For instructions, see https://docs.neptune.ai/api_token
 """
 
 
@@ -473,3 +522,49 @@ It looks like you tried to log a preview (incomplete) metric update for a step t
 the last fully committed (complete) update. Once a complete value is recorded, any preview updates
 must only be added for later steps. Please adjust the order of your updates and try again.
 """
+
+
+class NeptuneLocalStorageInUnsupportedVersion(NeptuneScaleError):
+    message = """The local storage database is in an unsupported version.
+    This may happen when you try to use a database created with a newer version of Neptune with an older version of the library.
+    Please either upgrade Neptune to the latest version or create a new local storage database."""
+
+
+class NeptuneDatabaseConflict(NeptuneScaleError):
+    message = """NeptuneDatabaseConflict: Database with the same name `{name}` already exists."""
+
+    def __init__(self, path: str = "") -> None:
+        super().__init__(name=path)
+
+
+class NeptuneHistogramBinEdgesContainNaN(NeptuneScaleError):
+    message = """
+{h1}
+NeptuneHistogramBinEdgesContainNaN: Histogram bin edges cannot contain NaN values.
+{end}
+"""
+
+
+class NeptuneHistogramTooManyBins(NeptuneScaleError):
+    message = """
+{h1}
+NeptuneHistogramTooManyBins: Histogram can have at most 512 bins
+{end}
+This corresponds to the maximum number of 513 bin edges.
+"""
+
+
+class NeptuneHistogramBinEdgesNotIncreasing(NeptuneScaleError):
+    message = """
+{h1}
+NeptuneHistogramBinEdgesNotIncreasing: Histogram bin edges must be strictly increasing.
+{end}
+"""
+
+
+class NeptuneHistogramValuesLengthMismatch(NeptuneScaleError):
+    message = """
+{h1}
+NeptuneHistogramValuesLengthMismatch: Histogram counts/densities length must be equal to bin edges length - 1.
+{end}
+"""
